/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbitkImageReader.h,v $
  Language:  C++
  Date:      $Date: 2010/04/08 14:37:58 $
  Version:   $Revision: 1.9 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         

/**
 * \file 
 * \brief class ITKImageReader : generic ITKImage reader
 */
/**
 * \class bbtk::ITKImageReader
 * \brief Generic ITKImage reader
 */
#ifdef _USE_ITK_

#ifndef __bbitkImageReader_h_INCLUDED__
#define __bbitkImageReader_h_INCLUDED__

#include "bbtkAtomicBlackBox.h"
#include "bbitkImage.h"

namespace bbitk
{

  //=================================================================
  // BlackBox declaration
  class ImageReader : public bbtk::AtomicBlackBox  
  {
    
    BBTK_BLACK_BOX_INTERFACE(ImageReader,
			     bbtk::AtomicBlackBox);

    BBTK_DECLARE_INPUT(In,std::string);
    BBTK_DECLARE_OUTPUT(Out, anyImagePointer);
    BBTK_PROCESS(Read);
    void Read();

  private:
    // Template read method 
    template<class itkImageType> void Read();
  };
  //=================================================================  

  //=================================================================
  // BlackBox description
  BBTK_BEGIN_DESCRIBE_BLACK_BOX(ImageReader,bbtk::AtomicBlackBox);
  BBTK_NAME("ImageReader");
  BBTK_AUTHOR("laurent.guigues at creatis.insa-lyon.fr");
  BBTK_CATEGORY("image; read/write");
  BBTK_DESCRIPTION("Generic itk image reader");
  BBTK_INPUT(ImageReader, In,
	     "filename with complete path", std::string,"file name");
  BBTK_OUTPUT(ImageReader, Out, "Output image" ,anyImagePointer,"");
  BBTK_END_DESCRIBE_BLACK_BOX(ImageReader);
  //=================================================================
  

}
//namespace bbtk
#endif

#endif
